#ifndef scores_h
#define scores_h
////////////////
////
//
///////////////////////////////////////////////////////////////////////
//																							//
//		SCORES.H	-	A system for managing Multiplayer game scoreboards	//
//						Developed for the SCION project.							//
//																							//
///////////////////////////////////////////////////////////////////////
//																							//
//		Author	:	EvilSOB															//
//		Written	: 	18-07-2010														//
//		Version	:	2.0																//
//																							//
//		Requirements:	Acknex.h														//
//																							//
///////////////////////////////////////////////////////////////////////
//																							//
//		Description:																	//
//																							//
//			This system (by default) automatically creates a scoreboard //
//		header called "User_Scores", containing a SCORE_TABLE struct.	//
//		This structure contains a list of USER_SCORE records, with a 	//
//		quantity specified by the #define "MAX_PLAYERS".					//
//		These records are persistant in their locations, and can			//
//		therefore have a pointer to themselves stored elsewhere			//
//		(for instance, in a entity skill) that will always be pointing	//
//		to the same 'player's details.											//
//			Again, because this data is persistant, any changes made		//
//		through this pointer will directly affect the database data.	//
//		BUT, these changes will only appear in the output TEXTs once	//
//		a sort has occured. (see "changed" notes later)						//
//			The 'sorting' process only affects the output TEXTs, being	//
//		"User_Scores.names", "User_Scores.kills", and the like, which 	//
//		have	pointers stored in the "User_Scores" header.					//
//		These TEXTs are individually controllable in their position,	//
//		font, color, etc.  BUT there are 'master' controls, available	//
//		via the header for layer, color, font, and scale.  				//
//		(note: scale is only applicable to bitmap fonts).					//
//		If these master controls are left, or set, to NULLs, then		//
//		the output TEXTs will retain whatever properties they were		//
//		last set to, or whatever you manually set them to later.			//
//			The sorting process is either triggered manually by calling	//
//		the "User_Scores_Sort" function, or automatically by modifying	//
//		scoreboard contents with one of the supplied "game_?" functions//
//		or alternatively by changing the "User_Scores.state" value.		//
//			The "User_Scores.state" is the controller for the order		//
//		that the output TEXTs take. (see "state" notes later)				//
//																							//
//																							//
//===================================================================//
//===================================================================//
//		NOTES:																			//
//===================================================================//
//===================================================================//
//																							//
//		"#define MAX_PLAYERS 16"													//
//		------------------------													//
//			This define specifies the MAXIMUM number of players that		//
//		will be tracked by the scoreboard system.								//
//		This script comes defaulted to 16.										//
//																							//
//===================================================================//
//																							//
//		"#define LOGON_AT"															//
//		------------------															//
//			This define specifies that the "logon" times recorded by		//
//		the scoreboard system will be displayed as the local				//
//		time-of-day that they were added to the scoreboard.				//
//		Comment out the #define to display the logon times as the		//
//		length of time they have been lodded in for.							//
//		The time display format is HH:MM:SS (in 24hr time)					//
//																							//
//===================================================================//
//																							//
//		"User_Scores.state"															//
//		-------------------															//
//			This variable is the primary control over the sort order		//
//		of the output TEXTs. They are denoted by the following:			//
//		1 = sort ascending by name													//
//		2 = sort ascending by team													//
//		3 = sort ascending by kills												//
//		4 = sort ascending by deaths												//
//		5 = sort ascending by score												//
//		And the same numbers as negatives denote descending order.		//
//		eg "-3" denotes to sort by descending 'kills' order.				//
//		------------------------------------------------------------	//
//		Special Note: Setting 'state' to NULL will terminate the 		//
//		startup function and free all memory used by the scoreboard.	//
//		This is NORMALLY only performed by the "User_Scores_startup"	//
//		function if it finds a scoreboard already exists.					//
//																							//
//===================================================================//
//																							//
//		"User_Scores.changed"														//
//		---------------------														//
//			This variable is used by the system to auto-sort on changes	//
//		or by you to trigger a refresh of the sort data.					//
//		If set to anything not NULL, an update/re-sort will occur 		//
//		during the NEXT frame. (and self-resets to NULL on execution)	//
//		(The "game_refresh" function performs the refresh THIS frame)	//
//																							//
//===================================================================//
//																							//
//		'User_Scores_startup()' function											//
//		--------------------------------											//
//			This function is the core-code of this scoreboard system.	//
//		It performs the auto-sorts if something has changed by calling	//
//		the "User_Scores_Sort" function if it detects any major data 	//
//		has changed, those being number-of-players and sort-state.		//
//		Calling this function a second time (or third etc) will cause	//
//		any pre-existing scoreboard to self-erase itself, as well as	//
//		a new one being created.													//
//																							//
//===================================================================//
//																							//
//		'User_Scores_Sort([state])' function									//
//		------------------------------------									//
//			This function is the core-sorting function. When called it 	//
//		sorts the data immediately. When called with a parameter of		//
//		type long, (see "User_Scores.state"), it will sort the data		//
//		into this order, and set the "User_Scores.state" variable to	//
//		this value. When called without a parameter, it performs an		//
//		immediate sort with the sort order "User_Scores.state".			//
//																							//
//===================================================================//
//===================================================================//
//																							//
//		'game_find(Name)'																//
//		-----------------																//
//			This function RETRIEVES an existing players "USER_SCORE"		//
//		structure, or else rreturns NULL.										//
//																							//
//===================================================================//
//																							//
//		'game_join(Name, Team [, Player_id])'									//
//		-------------------------------------									//
//			This function ADDS a new player to the scoreboard.				//
//		Name is truncated to 20 characters in length.						//
//		Team is a number of type long, no other limit.						//
//		Player_id is an optional number of type long, no other limit.	//
//		Function returns a NULL if the player couldnt be created, 		//
//		because either his name is blank, or the scoreboard is full,	//
//		or the name already exists, or the scoreboard doesnt.				//
//		------------------------------------------------------------	//
//		Special Note: On success, this function returns a pointer to	//
//		the "USER_SCORE" structure containing his information.			//
//		This structure is persistant in memory, and so can be used		//
//		later to access and modify his records directly, regardless		//
//		of his actual location within the sort order. Be aware that 	//
//		changing his data this way will not trigger am automatic			//
//		refresh of the	sort-data.													// 
//																							//
//===================================================================//
//																							//
//		'game_leave(Name)'															//
//		------------------															//
//			This function REMOVES an existing player from the board.		//
//		The function returns TRUE on a successful removal, otherwise	//
//		it returns FALSE, meaning the player-name was not found,	or		//
//		the scoreboard doesnt currently exist.									//
//																							//
//===================================================================//
//																							//
//		'game_zero()'																	//
//		-------------																	//
//			This functions RESETS all player scores to ZERO. These 		//
//		scores consist of 'kills', 'deaths', and 'score'.					//
//		Be aware that 'team', 'ping', and 'logged_on' are unchanged.	//
//																							//
//===================================================================//
//																							//
//		'game_clear()'																	//
//		--------------																	//
//			This functions CLEARS all players from the scoreboard.		//
//		All names and scores are removed but the scoreboard still		//
//		'exists', but it contains no players. 									//
//																							//
//===================================================================//
//																							//
//		'game_destroy()'																//
//		----------------																//
//			This function frees all memory and pointers used by the		//
//		scoreboard, and terminates its 'User_Scores_startup'.				//
//		This will occur NEXT frame.  So if you calling this in order	//
//		to create a new table THIS frame, then dont bother. Just call 	//
//		"User_Scores_startup" function and it will do it for you.		//
//																							//
//===================================================================//
//																							//
//		'game_refresh()'																//
//		----------------																//
//			This function forces an immediate sort of the data, in 		//
//		order to force an update of the sorted output TEXTs.				//
//		(this is the same as calling "User_Scores_Sort()")					//
//																							//
//===================================================================//
//																							//
//		'game_player_scores(Name, Kills, Deaths, Score)'					//
//		------------------------------------------------					//
//			This function UPDATEs the scores of the player named and 	//
//		also forces an update/re-sort of the scoreboard outputs.			//
//		If you put a number into Kills, Deaths, or Score, that number	//
//		is added to the players existing scores, conversely, if you  	//
//		put a negative number it will be subtracted from his scores.	//
//		And, as you can surmise, a zero changes nothing.					//
//		The function returns TRUE on a successful removal, otherwise	//
//		it returns FALSE, meaning the player-name was not found,	or		//
//		the scoreboard doesnt currently exist.									//
//		------------------------------------------------------------	//
//		Special Note: On success, this function returns a pointer to	//
//		the "USER_SCORE" structure containing his information.			//
//		See the "game_join" function above more more information.		//
//																							//
//===================================================================//
//																							//
//		'game_player_config(Name, Team, Ping, Player_id)'					//
//		------------------------------------------------					//
//		### NOT RECOMMENDED :: Use game_set_??? functions instead. ### //
//			This function SETs the configuration of the player named & 	//
//		also forces an update/re-sort of the scoreboard outputs.			//
//		ALL three values, Team, Ping, and Player_id, will be set to		//
//		the new values supplied, even when NULL, so be careful.			//
//		The function returns TRUE on a successful update, otherwise		//
//		it returns FALSE, meaning the player-name was not found,	or		//
//		the scoreboard doesnt currently exist.									//
//		------------------------------------------------------------	//
//		Special Note: On success, this function returns a pointer to	//
//		the "USER_SCORE" structure containing his information.			//
//		See the "game_join" function above more more information.		//
//																							//
//===================================================================//
//																							//
//		'game_set_team(Name, Team)'												//
//		---------------------------												//
//			This function SETs the players team to the new supplied		//
//		team and also forces an update/re-sort of the scoreboard.		//
//		The function returns TRUE on a successful update, otherwise		//
//		it returns FALSE, meaning the player-name was not found,	or		//
//		the scoreboard doesnt currently exist.									//
//		------------------------------------------------------------	//
//		Special Note: On success, this function returns a pointer to	//
//		the "USER_SCORE" structure containing his information.			//
//		See the "game_join" function above more more information.		//
//																							//
//===================================================================//
//																							//
//		'game_set_ping(Name, Ping)'												//
//		---------------------------												//
//			This function SETs the players ping to the new supplied		//
//		ping and also forces an update/re-sort of the scoreboard.		//
//		The function returns TRUE on a successful update, otherwise		//
//		it returns FALSE, meaning the player-name was not found,	or		//
//		the scoreboard doesnt currently exist.									//
//		------------------------------------------------------------	//
//		Special Note: On success, this function returns a pointer to	//
//		the "USER_SCORE" structure containing his information.			//
//		See the "game_join" function above more more information.		//
//																							//
//===================================================================//
//																							//
//		'game_set_playerid(Name, Player_id)'									//
//		------------------------------------									//
//			This function SETs the players Player_id to the supplied		//
//		Player_id and also forces an update/re-sort of the scoreboard.	//
//		The function returns TRUE on a successful update, otherwise		//
//		it returns FALSE, meaning the player-name was not found,	or		//
//		the scoreboard doesnt currently exist.									//
//		------------------------------------------------------------	//
//		Special Note: On success, this function returns a pointer to	//
//		the "USER_SCORE" structure containing his information.			//
//		See the "game_join" function above more more information.		//
//																							//
//===================================================================//
//																							//
//		FURTHER NOTES::																//
//																							//
//																							//
//																							//
//																							//
//																							//
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
//
//
#define MAX_PLAYERS 16
//
#define LOGON_AT			//comment out for logon FOR_HOW_LONG times
//
//
//
///////////////////////////////////////////////////////////////////////
typedef struct USER_SCORE
{	//SCORES database record structure
	long	player_id;		//optional unique player network ID
	long	sort_pos;		//sorted position with table	(NULL if un-used)
	char	name[21];		//player name (max 20 chars)
	long	team;				//team ID number
	long	kills;			//number of kills
	long	deaths;			//number of times killed
	long	score;			//total score of player
	long	logon;			//based on "total_ticks"
	long	ping;				//last recorded ping
}USER_SCORE;
//---------------------------------------------------------------------
typedef struct SCORE_TABLE
{	//SCORES database header/access structure.
	long	count;				//current player count
	USER_SCORE*	database;	//list of player scores
	TEXT*	names;				//sorted table, names column
	TEXT*	teams;				//sorted table, team column
	TEXT*	kills;				//sorted table, kills column
	TEXT*	deaths;				//sorted table, deaths column
	TEXT*	scores;				//sorted table, scores column
	TEXT*	pings;				//unsorted last-recorded-ping column
	TEXT*	logons;				//unsorted logon-time
	// Optional CONTROLS //
	long	layer;				//master layer for all texts, or NULL
	FONT*	font;					//master font for all texts,  or NULL
	var	blue,green,red;	//master color for all texts, or NULLs
	var	scale_x,scale_y;	//master scale for all texts, or NULLs
									//scale only effective on bitmapped fonts
	// CONTROLS ///////////	
	long	changed;	// table auto-sort/re-display trigger
	long	state;	// table states ::
						//	0 = TERMINATE - free all memory and terminate
						//	1 = sort ascending by name
						//	2 = sort ascending by team
						//	3 = sort ascending by kills
						//	4 = sort ascending by deaths
						//	5 = sort ascending by score
						// (negatives = descending)
}SCORE_TABLE;
//
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
SCORE_TABLE*	User_Scores=NULL;
//
void User_Scores_Sort(long state);		//manual re-sort prototype
//
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
void User_Scores_startup()
{	if(User_Scores)	User_Scores.state = NULL;
	User_Scores = sys_malloc(sizeof(SCORE_TABLE));
	memset(User_Scores,0,sizeof(SCORE_TABLE));
	User_Scores.database = sys_malloc(sizeof(USER_SCORE)*MAX_PLAYERS);
	memset(User_Scores.database,0,sizeof(USER_SCORE)*MAX_PLAYERS);
	TEXT* names  = User_Scores.names   = txt_create(MAX_PLAYERS,0);
	TEXT* teams  = User_Scores.teams   = txt_create(MAX_PLAYERS,0);
	TEXT* kills  = User_Scores.kills   = txt_create(MAX_PLAYERS,0);
	TEXT* deaths = User_Scores.deaths  = txt_create(MAX_PLAYERS,0);
	TEXT* scores = User_Scores.scores  = txt_create(MAX_PLAYERS,0);
	TEXT* pings  = User_Scores.pings   = txt_create(MAX_PLAYERS,0);
	TEXT* logons = User_Scores.logons  = txt_create(MAX_PLAYERS,0);
	USER_SCORE *SRC, *TARG;
	long _count, _state;	//auto-update triggers (previous values)
	_state = User_Scores.state	= 1;	//default to ascending names
	_count = User_Scores.count	= 0;	//no entries at this time
	User_Scores.changed			= 1;	//force initial update
	//------------------------------------------------------------------
	while(User_Scores.state)	//terminate on state == Zero
	{	if(User_Scores.changed)	//if update is required
		{	//update any display properties changes.----------------------
			if(User_Scores.layer)	//update TEXT layers?
			{	layer_sort(names,  User_Scores.layer);
				layer_sort(teams,  User_Scores.layer);
				layer_sort(kills,  User_Scores.layer);
				layer_sort(deaths, User_Scores.layer);
				layer_sort(scores, User_Scores.layer);	
				layer_sort(pings,  User_Scores.layer);	
				layer_sort(logons, User_Scores.layer);	}
			if(User_Scores.font)		//update TEXT fonts?
			{	names.font = User_Scores.font;
				teams.font = User_Scores.font;
				kills.font = User_Scores.font;
				deaths.font = User_Scores.font;
				scores.font = User_Scores.font;
				pings.font  = User_Scores.font;
				logons.font = User_Scores.font;	}
			if(vec_dist(User_Scores.blue, nullvector)) //update colors?
			{	vec_set(names.blue, User_Scores.blue);
				vec_set(teams.blue, User_Scores.blue);
				vec_set(kills.blue, User_Scores.blue);
				vec_set(deaths.blue, User_Scores.blue);
				vec_set(scores.blue, User_Scores.blue);
				vec_set(pings.blue, User_Scores.blue);
				vec_set(logons.blue, User_Scores.blue);	}
			if(User_Scores.scale_x||User_Scores.scale_y) //update scale?
			{	names.scale_x = User_Scores.scale_x;
				names.scale_y = User_Scores.scale_y;
				teams.scale_x = User_Scores.scale_x;
				teams.scale_y = User_Scores.scale_y;
				kills.scale_x = User_Scores.scale_x;
				kills.scale_y = User_Scores.scale_y;
				deaths.scale_x = User_Scores.scale_x;
				deaths.scale_y = User_Scores.scale_y;
				scores.scale_x = User_Scores.scale_x;
				scores.scale_y = User_Scores.scale_y;
				pings.scale_x = User_Scores.scale_x;
				pings.scale_y = User_Scores.scale_y;
				logons.scale_x = User_Scores.scale_x;
				logons.scale_y = User_Scores.scale_y;	}
			//re-sort tables----------------------------------------------
			User_Scores_Sort(User_Scores.state);
			//clean up----------------------------------------------------
			_state = User_Scores.state;	//new state applied
			_count = User_Scores.count;	//new player count applied
			User_Scores.changed = 0;		//all changes applied
		}
		wait(1);
		if((_state!=User_Scores.state)||(_count!=User_Scores.count))
		{	User_Scores.changed = 1;	}	//auto-sort on change
	}
	//------------------------------------------------------------------
	int i;	for(i=0; i<MAX_PLAYERS; i++)
	{	if(((User_Scores.names)->pstring)[i])
			str_remove(((User_Scores.names)->pstring)[i]);
		if(((User_Scores.teams)->pstring)[i])
			str_remove(((User_Scores.teams)->pstring)[i]);
		if(((User_Scores.kills)->pstring)[i])
			str_remove(((User_Scores.kills)->pstring)[i]);
		if(((User_Scores.deaths)->pstring)[i])
			str_remove(((User_Scores.deaths)->pstring)[i]);
		if(((User_Scores.scores)->pstring)[i])
			str_remove(((User_Scores.scores)->pstring)[i]);
		if(((User_Scores.pings)->pstring)[i])
			str_remove(((User_Scores.pings)->pstring)[i]);
		if(((User_Scores.logons)->pstring)[i])
			str_remove(((User_Scores.logons)->pstring)[i]);	}
	txt_remove(User_Scores.names);	txt_remove(User_Scores.teams);
	txt_remove(User_Scores.kills);	txt_remove(User_Scores.deaths);
	txt_remove(User_Scores.scores);	txt_remove(User_Scores.pings);
	txt_remove(User_Scores.logons);
	//------------------------------------------------------------------
	sys_free(User_Scores.database);	sys_free(User_Scores);
	User_Scores = NULL;					
}
//
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
STRING* logged_on_for(long raw_time)	//based on total_ticks
{	static STRING* time_str = NULL;
	if(!time_str)	time_str = str_create("");
	else				str_cpy(time_str, "");
	long on_for = (total_ticks - raw_time)/16;
	str_cat_num(time_str, "%02.0f:", cycle(on_for/60/60,0,24));
	str_cat_num(time_str, "%02.0f:", cycle(on_for/60,0,60));
	str_cat_num(time_str, "%02.0f",  cycle(on_for,0,60));
	return(time_str);														 }
//
//	
STRING* logged_on_at(long raw_time)		//based on total_ticks
{	static STRING* time_str = NULL;
	static long		eng_start = 0;
	if(!time_str)	time_str = str_create("");
	else				str_cpy(time_str, "");
	if(!eng_start)
	{	eng_start  = sys_seconds + sys_minutes*60;
		eng_start += sys_hours*60*60 - total_ticks/16;	}
	long on_at = raw_time/16 + eng_start;
	str_cat_num(time_str, "%02.0f:", cycle(on_at/60/60,0,24));
	str_cat_num(time_str, "%02.0f:", cycle(on_at/60,0,60));
	str_cat_num(time_str, "%02.0f",  cycle(on_at,0,60));
	return(time_str);														 }
//
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
void User_Scores_Sort(long state)
{	if(!User_Scores)		return;	//table not active
	if(state)	User_Scores.state = state;
	//------------------------------------------------------------------
	USER_SCORE *SRC, *TARG;
	TEXT *names = User_Scores.names;
	TEXT *teams = User_Scores.teams,		*kills  = User_Scores.kills;
	TEXT *deaths= User_Scores.deaths,	*scores = User_Scores.scores;
	TEXT *pings = User_Scores.pings,		*logons = User_Scores.logons;
	int c,s,t;	for(t=0; t<MAX_PLAYERS; t++)	//clear workspace
	{	(User_Scores.database)[t].sort_pos = 0;	}	
	for(t=1;	t<=User_Scores.count; t++)	//sort data
	{	TARG = NULL;	for(s=0;	s<MAX_PLAYERS; s++)
		{	if(!TARG)	{	TARG = &(User_Scores.database)[s];	
								if((!TARG.name[0])||(TARG.sort_pos))
									TARG=NULL;								}
			else
			{	SRC  = &(User_Scores.database)[s];
				if((SRC.name[0])&&(!SRC.sort_pos))
					switch((User_Scores.state))
					{	//Sort ASCENDING
						case	1:
							for(c=0; c<21; c++)	
							{	if(SRC.name[c]!=TARG.name[c])  break; }
							if(SRC.name[c]<TARG.name[c])		TARG=SRC;
						case  2:	if(SRC.team<TARG.team)		TARG=SRC;
						case  3:	if(SRC.kills<TARG.kills)	TARG=SRC;
						case  4:	if(SRC.deaths<TARG.deaths)	TARG=SRC;
						case  5:	if(SRC.score<TARG.score)	TARG=SRC;
						//Sort DESCENDING
						case -1:
							for(c=0; c<21; c++)	
							{	if(SRC.name[c]!=TARG.name[c])  break; }
							if(SRC.name[c]>TARG.name[c])		TARG=SRC;
						case -2:	if(SRC.team>TARG.team)		TARG=SRC;
						case -3:	if(SRC.kills>TARG.kills)	TARG=SRC;
						case -4:	if(SRC.deaths>TARG.deaths)	TARG=SRC;
						case -5:	if(SRC.score>TARG.score)	TARG=SRC;
		}	}		}
		TARG.sort_pos = t;													}
	//fill tables from sort result--------------------------------------
	for(t=0; t<MAX_PLAYERS; t++)	//clear text data
	{	str_cpy((names.pstring)[t],"");	str_cpy((teams.pstring)[t], "");
		str_cpy((kills.pstring)[t],"");	str_cpy((deaths.pstring)[t],"");
		str_cpy((scores.pstring)[t],"");	str_cpy((pings.pstring)[t], "");
		str_cpy((logons.pstring)[t],"");											 }
	for(s=0; s<MAX_PLAYERS; s++)
	{	SRC = &(User_Scores.database)[s];
		if(SRC.sort_pos)		//build texts from sorted data
		{	t = SRC.sort_pos-1;
			str_cpy((names.pstring)[t], SRC.name);
			str_cpy((teams.pstring)[t], str_for_int(0,SRC.team));
			str_cpy((kills.pstring)[t], str_for_int(0,SRC.kills));
			str_cpy((deaths.pstring)[t],str_for_int(0,SRC.deaths));
			str_cpy((scores.pstring)[t],str_for_int(0,SRC.score));
			str_cpy((pings.pstring)[t],str_for_int(0,SRC.ping));
#ifdef LOGON_AT
			str_cpy((logons.pstring)[t],logged_on_at(SRC.logon));
#else
			str_cpy((logons.pstring)[t],logged_on_for(SRC.logon));
#endif
			SRC.sort_pos = s+1;											}	}
}
void User_Scores_Sort()	{	User_Scores_Sort(User_Scores.state);	}
//
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
//
//
//
//=====================================================================
//=====================================================================
//		SCORBOARD MANAGEMENT FUNCTIONS
//=====================================================================
//=====================================================================
//
//	*******************************************************************
//	FUNCTIONS - Find Player (by name)
//	*******************************************************************
USER_SCORE* game_find(STRING* name)
{	//function returns TRUE if player found sucessfully, else ULL
	if(!name)					return(0);	//invalid player name
	USER_SCORE* SRC;			STRING* _name = str_create(_chr(name));
	int t,c;		for(t=0; t<MAX_PLAYERS;	t++)
	{	SRC = &(User_Scores.database)[t];	
		for(c=0;c<(str_len(_name)+1);c++)
		{	if(SRC.name[c]!= (_name.chars)[c])	break;	}
		if(c>(str_len(_name)))		break;	 				}
	str_remove(_name);	if(t>=MAX_PLAYERS)	return(0);
	//------------------------------------------------------------------
	return(SRC);	//return SUCCESSFULLY found player
}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Join
//	*******************************************************************
USER_SCORE* game_join(STRING* name, int team, int player_id)
{	//function returns USER_SCORE if player added sucessfully,else zero
	if(!name)							return(NULL);	//invalid player name
	if(User_Scores.count>=MAX_PLAYERS)	return(NULL);	//table full
	if(game_find(name))		return(NULL);	//player name already exists
	int i;	for(i=0; i<MAX_PLAYERS;	i++)
		if(!(User_Scores.database)[i].name[0])		break;
	if(i>=MAX_PLAYERS)		return(NULL);	//un-expectedly full ?!?
	//------------------------------------------------------------------
	User_Scores.count++;
	USER_SCORE* SRC = &(User_Scores.database)[i];
	memset(SRC, 0, sizeof(USER_SCORE));
	memcpy(SRC.name, _chr(name), 20);	
	SRC.team = team;	SRC.player_id = player_id;
	SRC.logon = total_ticks;
	return(SRC);	//return pointer to player data
}
USER_SCORE* game_join(STRING* n,int t)	{return(game_join(n,t,0));}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Leave
//	*******************************************************************
function game_leave(STRING* name)
{	//function returns TRUE if player removed sucessfully, else zero
	if(!name)					return(0);	//invalid player name
	USER_SCORE* SRC = game_find(name);
	if(!SRC)						return(0);	//player name not found
	//------------------------------------------------------------------
	memset(SRC, 0, sizeof(USER_SCORE));		User_Scores.count--;
	return(1);	//return SUCCESS
}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Reset All Player Scores
//	*******************************************************************
void game_zero()
{	USER_SCORE* SRC;	int t;	for(t=0; t<MAX_PLAYERS;	t++)
	{	SRC = &(User_Scores.database)[t];	
		SRC.kills = SRC.deaths = SRC.score = 0;	}//reset player scores
	User_Scores.changed = 1;	//force update
}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Remove All Players
//	*******************************************************************
void game_clear()
{	USER_SCORE* SRC;	int t;	for(t=0; t<MAX_PLAYERS;	t++)
		memset((User_Scores.database)[t], 0, sizeof(USER_SCORE));
	User_Scores.count = 0;
	User_Scores.changed = 1;	//force update
}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Remove Entire Table and free memory.
//	*******************************************************************
void game_destroy()	{	User_Scores.state = NULL;	}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Refresh all Player Data
//	*******************************************************************
void game_refresh()	{	User_Scores_Sort();	}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Update Player SCORE Data
//	*******************************************************************
USER_SCORE* game_player_scores(STRING* name,int kills,int deaths,int score)
{	//function returns USER_SCORE if player updated sucessfully, else zero
	if(!name)					return(0);	//invalid player name
	USER_SCORE* SRC = game_find(name);
	if(!SRC)						return(0);	//player name not found
	//------------------------------------------------------------------
	SRC.kills+=kills;		SRC.deaths+=deaths;		SRC.score+=score;
	User_Scores.changed = 1;	//force update
	return(SRC);	//return pointer to player data
}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Update Player CONFIG Data
//	*******************************************************************
USER_SCORE* game_player_config(STRING* name,int team,int ping,int player_id)
{	//function returns USER_SCORE if player updated sucessfully, else zero
	if(!name)					return(0);	//invalid player name
	USER_SCORE* SRC = game_find(name);
	if(!SRC)						return(0);	//player name not found
	//------------------------------------------------------------------
	SRC.team=team;		SRC.ping=ping;		SRC.player_id=player_id;
	User_Scores.changed = 1;	//force update
	return(SRC);	//return pointer to player data
}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Update Player TEAM Data
//	*******************************************************************
USER_SCORE* game_set_team(STRING* name, long team)
{	//function returns USER_SCORE if player updated sucessfully, else zero
	if(!name)					return(0);	//invalid player name
	USER_SCORE* SRC = game_find(name);
	if(!SRC)						return(0);	//player name not found
	//------------------------------------------------------------------
	SRC.team=team;		User_Scores.changed = 1;	return(SRC);			}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Update Player PING Data
//	*******************************************************************
USER_SCORE* game_set_ping(STRING* name, long ping)
{	//function returns USER_SCORE if player updated sucessfully, else zero
	if(!name)					return(0);	//invalid player name
	USER_SCORE* SRC = game_find(name);
	if(!SRC)						return(0);	//player name not found
	//------------------------------------------------------------------
	SRC.ping=ping;		User_Scores.changed = 1;	return(SRC);			}
//
//
//
//	*******************************************************************
//	FUNCTIONS - Update Player Player_ID Data
//	*******************************************************************
USER_SCORE* game_set_playerid(STRING* name, long player_id)
{	//function returns USER_SCORE if player updated sucessfully, else zero
	if(!name)					return(0);	//invalid player name
	USER_SCORE* SRC = game_find(name);
	if(!SRC)						return(0);	//player name not found
	//------------------------------------------------------------------
	SRC.player_id=player_id;	User_Scores.changed=1;	return(SRC);	}
//
//
//
//
//
////
////////////////////
#endif	//scores_h
